@@ -16,11 +16,11 @@ import tarfile
 import io
 import time
 import os
+import psutil
 
 # yes, python 3.2 has exist_ok, but it will still fail if the mode is different
 
-whiskermenu_file = 'whiskermenu-1.rc'
-whiskermenu_path = os.path.join(GLib.get_home_dir(), '.config/xfce4/panel/', whiskermenu_file)
+config_dir = os.path.join(GLib.get_user_config_dir(), 'xfce4/panel/')
 
 def mkdir_p(path):
     try:
@@ -42,8 +42,9 @@ class PanelConfig(object):
     def __init__(self):
         self.desktops = []
         self.properties = {}
-        self.whiskermenu_data = ""
+        self.rc_files = []
         self.source = None
+        self.errors = []
 
     @classmethod
     def from_xfconf(cls, xfconf):
@@ -67,10 +68,8 @@ class PanelConfig(object):
 
         pc.remove_orphans()
         pc.find_desktops()
-        
-        if pc.check_whiskermenu():
-            pc.whiskermenu_data = pc.get_whiskermenu_file().read()
-       
+        pc.find_rc_files()
+
         return pc
 
     @classmethod
@@ -89,12 +88,13 @@ class PanelConfig(object):
 
         pc.remove_orphans()
         pc.find_desktops()
+        pc.find_rc_files()
 
-        if pc.check_whiskermenu():
-            pc.whiskermenu_data = pc.get_whiskermenu_file().read()
-        
         return pc
 
+    def source_not_file(self):
+        return getattr(self, 'source', None) is None
+
     def remove_orphans(self):
         plugin_ids = set()
         rem_keys = []
@@ -128,28 +128,17 @@ class PanelConfig(object):
 
         # Check if binary exists
         keyfile = GLib.KeyFile.new()
-        decoded = bytes.decode()
-        if keyfile.load_from_data(decoded, len(decoded),
-                                  GLib.KeyFileFlags.NONE):
-            try:
+        try:
+            if keyfile.load_from_data(bytes.decode(), len(bytes), GLib.KeyFileFlags.NONE):
                 exec_str = keyfile.get_string("Desktop Entry", "Exec")
                 if self.check_exec(exec_str):
                     return True
-            except GLib.Error:  # pylint: disable=E0712
-                pass #  https://bugzilla.xfce.org/show_bug.cgi?id=14597
+        except GLib.Error:  # pylint: disable=E0712
+            self.errors.append('Error parsing desktop file ' + path)
+            pass #  https://bugzilla.xfce.org/show_bug.cgi?id=14597
 
         return False
 
-    def check_whiskermenu(self):
-        try:
-            f = self.get_whiskermenu_file()
-            f.close()
-
-        except (KeyError, FileNotFoundError):
-            return False
-
-        return True
-
     def find_desktops(self):
         rem_keys = []
 
@@ -160,8 +149,11 @@ class PanelConfig(object):
                 number = path[2].split('-')[1]
                 if pv.get_type_string() == 's' and \
                         pv.get_string() == 'launcher':
-                    for d in self.properties['/plugins/plugin-' + number +
-                                             '/items'].unpack():
+                    prop_path = '/plugins/plugin-' + number + '/items'
+                    if prop_path not in self.properties:
+                        rem_keys.append('/plugins/plugin-' + number)
+                        continue
+                    for d in self.properties[prop_path].unpack():
                         desktop_path = 'launcher-' + number + '/' + d
                         if self.check_desktop(desktop_path):
                             self.desktops.append(desktop_path)
@@ -170,6 +162,41 @@ class PanelConfig(object):
 
         self.remove_keys(rem_keys)
 
+    def find_rc_files(self):
+        if self.source_not_file():
+            plugin_ids = []
+            filenames = []
+
+            for pp in self.properties:
+                path = pp.split('/')
+                if len(path) == 4 and path[0] == '' and path[1] == 'panels' and path[3] == 'plugin-ids':
+                    plugin_ids.extend(self.properties[pp])
+
+            if len(plugin_ids) == 0:
+                return
+
+            for plugin_id in plugin_ids:
+                plugin_id = str(plugin_id)
+                prop_path = '/plugins/plugin-' + plugin_id
+                try:
+                    prop = self.properties[prop_path].get_string()
+                except:
+                    continue
+
+                if prop == 'launcher':
+                    continue
+
+                filename = prop + '-' + plugin_id + '.rc'
+                path = os.path.join(config_dir, filename)
+                if os.path.exists(path):
+                    filenames.append(filename)
+            self.rc_files = filenames
+        else:
+            filenames = self.source.getnames()
+            for filename in filenames:
+                if filename.find('.rc') > -1:
+                    self.rc_files.append(filename)
+
     def remove_keys(self, rem_keys):
         keys = list(self.properties.keys())
         for param in keys:
@@ -181,19 +208,18 @@ class PanelConfig(object):
                         pass #  https://bugzilla.xfce.org/show_bug.cgi?id=14934
 
     def get_desktop_source_file(self, desktop):
-        if getattr(self, 'source', None) is None:
-            path = os.path.join(
-                GLib.get_home_dir(), '.config/xfce4/panel/', desktop)
+        if self.source_not_file():
+            path = os.path.join(config_dir, desktop)
             return open(path, 'rb')
         else:
             return self.source.extractfile(desktop)
 
-    def get_whiskermenu_file(self):
-        if getattr(self, 'source', None) is None:
-            return open(whiskermenu_path, 'rb')
-
+    def get_rc_source_file(self, rc):
+        if self.source_not_file():
+            path = os.path.join(config_dir, rc)
+            return open(path, 'rb')
         else:
-            return self.source.extractfile(whiskermenu_file)
+            return self.source.extractfile(rc)
 
     def to_file(self, filename):
         if filename.endswith('.gz'):
@@ -211,10 +237,10 @@ class PanelConfig(object):
         for d in self.desktops:
             bytes = self.get_desktop_source_file(d).read()
             add_to_tar(t, bytes, d)
-    
-        if self.check_whiskermenu():
-            bytes = self.get_whiskermenu_file().read()
-            add_to_tar(t, bytes, whiskermenu_file)
+
+        for rc in self.rc_files:
+            bytes = self.get_rc_source_file(rc).read()
+            add_to_tar(t, bytes, rc)
 
         t.close()
 
@@ -257,21 +283,33 @@ class PanelConfig(object):
                 xfconf.call_sync('SetProperty', GLib.Variant(
                     '(ssv)', ('xfce4-panel', pp, pv)), 0, -1, None)
 
-            panel_path = os.path.join(
-                GLib.get_home_dir(), '.config/xfce4/panel/')
             for d in self.desktops:
                 bytes = self.get_desktop_source_file(d).read()
-                mkdir_p(panel_path + os.path.dirname(d))
-                f = open(panel_path + d, 'wb')
+                mkdir_p(config_dir + os.path.dirname(d))
+                f = open(config_dir + d, 'wb')
                 f.write(bytes)
                 f.close()
 
+            for rc in self.rc_files:
+                bytes = self.get_rc_source_file(rc).read()
+                f = open(os.path.join(config_dir, rc), 'wb')
+                f.write(bytes)
+                f.close()
+
+                # Kill the plugin so that it reloads the config we just wrote and does
+                # not overwrite it with its current cache when the panel restarts below.
+                # Some plugins don't save their config when restarting the panel
+                # (e.g. whiskermenu) but others do (e.g. netload)
+                plugin_id = rc.replace('.', '-').split('-')[1]
+                proc_name_prefix = 'panel-' + plugin_id + '-'
+                for proc in psutil.process_iter():
+                    if proc.name().startswith(proc_name_prefix):
+                        proc.kill()
+
             try:
                 dbus_proxy.call_sync('Terminate', GLib.Variant('(b)', ('xfce4-panel',)), 0, -1, None)
             except GLib.GError:  # pylint: disable=E0712
                 pass
 
-        if self.check_whiskermenu():
-            f = open(whiskermenu_path, 'wb')
-            f.write(self.whiskermenu_data)
-            f.close()
+    def has_errors(self):
+        return len(self.errors) > 0
