@@ -20,11 +20,12 @@ import tarfile
 
 from locale import gettext as _
 
-import shlex
 import os
 import datetime
 
 import warnings
+import argparse
+import textwrap
 
 import gi
 gi.require_version('Gtk', '3.0')
@@ -54,12 +55,8 @@ class XfcePanelProfiles:
     data_dir = "xfce4-panel-profiles"
     save_location = os.path.join(GLib.get_user_data_dir(), data_dir)
 
-    def __init__(self, from_panel=False):
-        '''Initialize the Panel Profiles application.
-
-        If 'from_panel' is set to 'True' the application launches 'xfce4-panel
-        --preferences' when the user closes this application.
-        '''
+    def __init__(self):
+        '''Initialize the Panel Profiles application.'''
         # Temporary fix: https://stackoverflow.com/a/44230815
         _ = libxfce4ui.TitledDialog()
 
@@ -92,30 +89,11 @@ class XfcePanelProfiles:
         if not os.path.exists(self.save_location):
             os.makedirs(self.save_location)
 
-        self.from_panel = from_panel
-
         self.window.show()
 
     def _copy(self, src, dst):
         PanelConfig.from_file(src).to_file(dst)
 
-    def _filedlg(self, title, action, default=None):
-        if action == Gtk.FileChooserAction.SAVE:
-            button = _("Save")
-        else:
-            button = _("Open")
-        dialog = Gtk.FileChooserDialog(title=title,
-                                       transient_for=self.window,
-                                       action=action)
-        dialog.add_buttons(
-            _("Cancel"), Gtk.ResponseType.CANCEL,
-            button, Gtk.ResponseType.ACCEPT
-        )
-        dialog.set_default_response(Gtk.ResponseType.ACCEPT)
-        if default:
-            dialog.set_current_name(default)
-        return dialog
-
     def load_xfconf(self):
         session_bus = Gio.BusType.SESSION
         cancellable = None
@@ -242,12 +220,12 @@ class XfcePanelProfiles:
         dialog.destroy()
 
     def on_export_clicked(self, widget):
-        dialog = self._filedlg(_("Export configuration as..."),
-                               Gtk.FileChooserAction.SAVE, _("Untitled"))
+        dialog = PanelExportDialog(self.window)
         response = dialog.run()
         if response == Gtk.ResponseType.ACCEPT:
             selected = self.get_selected_filename()
-            filename = dialog.get_filename()
+            # The `.tar.bz2` suffix will be added in `{save,copy}_configuration`.
+            filename = os.path.join(dialog.file_chooser_button.get_filename(), dialog.entry_filename.get_text())
             if selected == "": # Current configuration.
                 self.save_configuration(filename, False)
             else:
@@ -255,8 +233,12 @@ class XfcePanelProfiles:
         dialog.destroy()
 
     def on_import_clicked(self, widget):
-        dialog = self._filedlg(_("Import configuration file..."),
-                               Gtk.FileChooserAction.OPEN)
+        dialog = Gtk.FileChooserDialog(title=_("Import configuration file..."),
+                                       transient_for=self.window,
+                                       action=Gtk.FileChooserAction.OPEN)
+        dialog.add_buttons(_("Cancel"), Gtk.ResponseType.CANCEL,
+                           _("Open"), Gtk.ResponseType.ACCEPT)
+        dialog.set_default_response(Gtk.ResponseType.ACCEPT)
         response = dialog.run()
         if response == Gtk.ResponseType.ACCEPT:
             filename = dialog.get_filename()
@@ -326,17 +308,7 @@ class XfcePanelProfiles:
         self.on_close_clicked(args)
 
     def on_close_clicked(self, *args):
-        '''
-        Exit the application when the window is closed. Optionally launch
-        'xfce4-panel --preferences' if the application is launched with
-        '--from-profile' option.
-        '''
-        if self.from_panel:
-            path = GLib.find_program_in_path('xfce4-panel')
-
-            if path != None:
-                GLib.spawn_command_line_async(path + ' --preferences')
-
+        '''Exit the application when the window is closed.'''
         Gtk.main_quit()
 
     def on_help_clicked(self, *args):
@@ -418,7 +390,7 @@ class PanelErrorDialog(Gtk.MessageDialog):
     configuration will be lost.'''
 
     def __init__(self, parent=None, messages=[]):
-        message = _("Errors occured while parsing the current configuration.")
+        message = _("Errors occurred while parsing the current configuration.")
 
         Gtk.MessageDialog.__init__(
             self, transient_for=parent, modal=True,
@@ -444,9 +416,77 @@ class PanelErrorDialog(Gtk.MessageDialog):
 
         box.show_all()
 
-if __name__ == "__main__":
-    from_panel = False
+class PanelExportDialog(Gtk.Dialog):
+    def __init__(self, parent=None):
+        Gtk.Dialog.__init__(self, title=_("Export configuration as..."), transient_for=parent)
+        self.set_default_size(400, 150)
 
+        box = self.get_content_area()
+        box.set_spacing(6)
+
+        label_filename = Gtk.Label(label=_("Filename"))
+        label_filename.set_xalign(0)
+        box.pack_start(label_filename, False, False, 0)
+
+        box_filename = Gtk.Box(spacing=6)
+        self.entry_filename = Gtk.Entry()
+        self.entry_filename.set_text(_("Untitled"))
+
+        label_extension = Gtk.Label(label=".tar.bz2")
+
+        box_filename.pack_start(self.entry_filename, True, True, 0)
+        box_filename.pack_start(label_extension, False, False, 0)
+        box.pack_start(box_filename, False, False, 0)
+
+        label_location = Gtk.Label(label=_("Location"))
+        label_location.set_xalign(0)
+        box.pack_start(label_location, False, False, 0)
+
+        self.file_chooser_button = Gtk.FileChooserButton(title=_("Select a Folder"))
+        self.file_chooser_button.set_action(Gtk.FileChooserAction.SELECT_FOLDER)
+        self.file_chooser_button.set_current_folder(os.path.expanduser("~"))
+        box.pack_start(self.file_chooser_button, False, False, 0)
+
+        button_box = Gtk.ButtonBox.new(Gtk.Orientation.HORIZONTAL)
+        button_box.set_spacing(6)
+        button_box.set_layout(Gtk.ButtonBoxStyle.END)
+
+        button_cancel = Gtk.Button(label=_("Cancel"))
+        button_cancel.connect("clicked", self.on_cancel_clicked)
+        button_box.add(button_cancel)
+
+        button_save = Gtk.Button(label=_("Save"))
+        button_save.connect("clicked", self.on_save_clicked)
+        button_box.add(button_save)
+
+        box.pack_end(button_box, False, False, 0)
+        self.show_all()
+
+    def on_cancel_clicked(self, widget):
+        self.response(Gtk.ResponseType.CANCEL)
+
+    def on_save_clicked(self, widget):
+        dest_dir = self.file_chooser_button.get_filename()
+        dest_name = self.entry_filename.get_text() + ".tar.bz2"
+        filename = os.path.join(dest_dir, dest_name)
+
+        can_export = True
+        if os.path.exists(filename):
+            confirm_overwrite = Gtk.MessageDialog(transient_for=self, message_type=Gtk.MessageType.QUESTION)
+
+            confirm_overwrite.set_markup(_('<b>A file named "%s" already exists. Do you want to replace it?</b>') % dest_name)
+            confirm_overwrite.format_secondary_text(_('The file already exists in "%s", replacing it will overwrite its contents.') % dest_dir)
+            confirm_overwrite.add_buttons(_("Cancel"), Gtk.ResponseType.CANCEL,
+                                          _("Replace"), Gtk.ResponseType.OK)
+
+            if confirm_overwrite.run() != Gtk.ResponseType.OK:
+                can_export = False
+            confirm_overwrite.destroy()
+
+        if can_export == True:
+            self.response(Gtk.ResponseType.ACCEPT)
+
+if __name__ == "__main__":
     import sys
 
     libxfce4util.textdomain('xfce4-panel-profiles',
@@ -472,31 +512,39 @@ if __name__ == "__main__":
         interface,
         cancellable)
 
-    if len(sys.argv) > 1:
-        if sys.argv[1] in ['save', 'load']:
-            try:
-                if sys.argv[1] == 'save':
-                    PanelConfig.from_xfconf(xfconf).to_file(sys.argv[2])
-                elif sys.argv[1] == 'load':
-                    PanelConfig.from_file(sys.argv[2]).to_xfconf(xfconf)
-            except Exception as e:
-                print(repr(e))
-                exit(1)
-            exit(0)
-        elif sys.argv[1] == '--version':
-            print(info.appname + ' ' + info.version)
-            exit(0)
-        elif sys.argv[1] == '--from-panel':
-            from_panel = True
-        else:
-            print('Xfce Panel Profiles - Usage:')
-            print(info.appname + ' : load graphical user interface.')
-            print(info.appname + ' save <filename> : save current configuration.')
-            print(info.appname + ' load <filename> : load configuration from file.')
-            print('')
-            exit(-1)
+    parser = argparse.ArgumentParser(
+        formatter_class=argparse.RawDescriptionHelpFormatter,
+        description=textwrap.dedent(f"""\
+            Xfce Panel Profiles {info.version}
 
-    main = XfcePanelProfiles(from_panel)
+            usage:
+              {info.appname}                  load graphical user interface
+              {info.appname} save <filename>  save current configuration
+              {info.appname} load <filename>  load configuration from file
+        """),
+        usage=argparse.SUPPRESS
+    )
+    parser.add_argument('--version', action='version', version=f"{info.appname} {info.version}")
+    subparsers = parser.add_subparsers(dest='subcommand', help=argparse.SUPPRESS)
+    save_parser = subparsers.add_parser('save')
+    save_parser.add_argument('filename', help='filename to save configuration')
+    load_parser = subparsers.add_parser('load')
+    load_parser.add_argument('filename', help='filename to load configuration')
+
+    args = parser.parse_args()
+
+    try:
+        if args.subcommand == 'save':
+            PanelConfig.from_xfconf(xfconf).to_file(args.filename)
+            exit(0)
+        elif args.subcommand == 'load':
+            PanelConfig.from_file(args.filename).to_xfconf(xfconf)
+            exit(0)
+    except Exception as e:
+        print(f"Error processing '{args.filename}': {repr(e)}")
+        exit(1)
+
+    main = XfcePanelProfiles()
 
     try:
         Gtk.main()
